local max_res = RenderSettings.modes[#RenderSettings.modes]
local function res_allowed_fullscreen(res)
	local allowed = false
	for _, allowed_res in pairs(RenderSettings.modes) do
		if res[1] == allowed_res.x and res[2] == allowed_res.y then
			allowed = true
		end
	end
	return allowed
end

local function set_windowed(windowed, mode)
	if windowed then
		managers.viewport:set_fullscreen(false)
	end
	
	if FullscreenWindowed and mode then
		FullscreenWindowed.library.change_display_mode(mode, RenderSettings.resolution.x, RenderSettings.resolution.y, RenderSettings.adapter_index)
		FullscreenWindowed._settings.display_mode = mode
		FullscreenWindowed:save_settings()
	end
end

local function create_dialog(title, text, buttons)
	local dialog = {
		title = managers.localization:text(title),
		text = managers.localization:text(text),
		button_list = buttons
	}
	
	managers.system_menu:show(dialog)
end

Hooks:Add("LocalizationManagerPostInit", "RESCHANGE_loc", function(...)
	LocalizationManager:add_localized_strings({
		menu_display_aspect_ratio = "Display Aspect Ratio",
		menu_custom_resoultion = "Custom",
		x_res_change = "Width",
		y_res_change = "Height",
	
		menu_display_mode = "Display Mode",
		menu_windowed = "Windowed",
		menu_fullscreen_windowed = "Fullscreen Windowed",
		menu_pixel_art = "Warning: Pixel art. Barely readable fonts.",
		menu_pixel_stew = "Warning: Pixel stew. Unreadable fonts.",
		menu_not_allowed_full_text = "This resolution is not supported on fullscreen display mode.\n\nYou have to change display mode manually or select one of the options down below.",
		menu_fullscreen_allowed = "Fullscreen is allowed on this resolution.",
		menu_extremely_low_resolution_title = "Extremely low resolution",
		menu_extremely_low_resolution_text = "You faced an extremely low screen resolution.\n\nThis resolution is only supported on fullscreen windowed display mode.\n\nIf something goes wrong you have to be sure that you can fix it manually.\n\nIf you couldn't change resolution in game, you have to delete ''renderer_settings.xml'' on this path:\n\n''C:/Users/USERNAME/AppData/Local/PAYDAY 2''\n\nThat will restore default settings, if you can't go through the options because of unreadable fonts.\n\nIf you couldn't find this file then Get The F. Out of here ASAP.",
		menu_ye_fo_sho_text = "Are you absolutely sure?",
		
		dialog_open_page = "Open page",
		download_fullwind = "Download ''Fullscreen Windowed'' first",
		download_fullwind_desc = "Click the button to enter the web page of the mod in the ModWorkshop",
	})

	if Idstring("russian"):key() == SystemInfo:language():key() then
		LocalizationManager:add_localized_strings({
			menu_display_aspect_ratio = "Соотношение сторон",
			menu_custom_resoultion = "Пользовательское",
			x_res_change = "Ширина",
			y_res_change = "Высота",
		
			menu_display_mode = "Режим дисплея",
			menu_windowed = "Оконный",
			menu_fullscreen_windowed = "Полный экран в окне",
			menu_pixel_art = "Внимание: пиксель-арт. Шрифт едва разлечим.",
			menu_pixel_stew = "Внимание: пиксельный суп. Шрифт неразлечим.",
			menu_not_allowed_full_text = "Это разрешение экрана недоступно в полноэкранном режиме.\n\nВы должны изменить настройки дисплея вручную или выбрать одну из опций внизу.",
			menu_fullscreen_allowed = "Поддерживает полноэкранный режим дисплея.",
			menu_extremely_low_resolution_title = "Экстримально низкое разрешение",
			menu_extremely_low_resolution_text = "Вы столкнулись с экстимально низким разрешением экрана.\n\nЭто разрешение экрана доступно только в полном экране в окне.\nЕсли что-то пойдет не так, то вы должны быть уверены что вы сможете исправить это вручную.\n\nЕсли вы не сможете восстановить старое разрешение экрана в игре, вам нужно удалить ''renderer_settings.xml'' по этому пути:\n\n''C:/Users/USERNAME/AppData/Local/PAYDAY 2''\n\nЭто восстановит ваши настройки по умолчанию, если вы не можете зайти в настройки разрешения из-за нечиаемого шрифта.\n\nЕсли вы не знаеете как найти этот файл, уберайтесь отсюда как можно скорее.",
			menu_ye_fo_sho_text = "Вы абсолютно уверены?",
	
			dialog_open_page = "Открыть страницу",
			download_fullwind = "Сначала скачайте Fullscreen Windowed",
			download_fullwind_desc = "Это разрешение экрана недоступно в полноэкранном режиме.\n\nКликни кнопку, чтобы перейти на страницу мода в ModWorkshop",
		})
	end
end)

Hooks:Add("MenuManagerInitialize", "MenuManagerInitialize_RESCHANGE", function(...)
	DelayedCalls:Add("DelayedCalls_set_fullwindow", 0.5, function()
		if FullscreenWindowed then
			local check_on_start = (RenderSettings.resolution.x < 480 or RenderSettings.resolution.y < 480) or FullscreenWindowed._settings.display_mode == 2
			set_windowed(check_on_start, check_on_start and 2)
		end
	end)
end)

function MenuCallbackHandler:change_resolution_new(item)
	local resolution = item:parameters().resolution
	
	local full = managers.viewport:is_fullscreen()
	local pixel_stew = resolution.x < 480 or resolution.y < 480
	
	if not FullscreenWindowed and pixel_stew then
		create_dialog("download_fullwind", "download_fullwind_desc", {
			{text = managers.localization:text("dialog_open_page"), callback_func = function()
				os.execute("start https://modworkshop.net/mod/27683")
			end},
			{text = managers.localization:text("menu_back"), cancel_button = true}
		})
		return
	end
	
	local res = {
		resolution.x,
		resolution.y
	}
	
	local function change_res(set_fullwindow)
		set_windowed(full and not res_allowed_fullscreen(res), set_fullwindow)
		
		local old_resolution = RenderSettings.resolution
		if resolution == old_resolution then
			return
		end

		managers.viewport:set_resolution(resolution)
		managers.viewport:set_aspect_ratio(resolution.x / resolution.y)

		local function on_decline()
			managers.viewport:set_resolution(old_resolution)
			managers.viewport:set_aspect_ratio(old_resolution.x / old_resolution.y)
		end
		
		managers.menu:show_accept_res_dialog(on_decline)
	end
	
	if pixel_stew then
		if RenderSettings.resolution.x >= 480 or RenderSettings.resolution.y >= 480 then
			create_dialog("menu_extremely_low_resolution_title", "menu_extremely_low_resolution_text", {
				{text = managers.localization:text("dialog_accept"), callback_func = function()
					create_dialog("menu_extremely_low_resolution_title", "menu_ye_fo_sho_text", {
						{text = managers.localization:text("dialog_yes"), callback_func = function()
							change_res(2)
						end},
						{text = managers.localization:text("dialog_no"), cancel_button = true}
					})
				end},
				{},
				{text = managers.localization:text("menu_back"), cancel_button = true}
			})
		else
			change_res(2)
		end
	elseif full and not res_allowed_fullscreen(res) then
		local buttons = {
			{text = managers.localization:text("menu_windowed"), callback_func = function()	
				change_res(1)
			end},
			{},
			{text = managers.localization:text("menu_back"), cancel_button = true}
		}
		if FullscreenWindowed then
			table.insert(buttons, 1, {text = managers.localization:text("menu_fullscreen_windowed"), callback_func = function()	
				change_res(2)
			end})
		end
		create_dialog("menu_display_mode", "menu_not_allowed_full_text", buttons)
	else
		change_res()
	end
end

function MenuResolutionCreator:modify_node(node)
	local new_node = deep_clone(node)
	
	local function add_divider(name, size)
		if not new_node:item(name) then
			local data = {
				name = name,
				type = "MenuItemDivider",
				size = size,
				no_text = true
			}
			
			local new_item = new_node:create_item(data, nil)
			new_node:add_item(new_item)
		end
	end

	if not new_node:item("res_aspect_ratio") then
		MenuCallbackHandler.res_aspect_ratio = function(self, item)
			if item:value() == 1 then
				Global.crimenet.res_aspect_ratio = nil
			else
				Global.crimenet.res_aspect_ratio = item:value()
			end
		end
		
		local params = {
			name = "res_aspect_ratio",
			callback = "res_aspect_ratio",
			text_id = "menu_display_aspect_ratio"
		}
		
		local data = {
			{
				value = 1,
				ratio = nil,
				text_id = "menu_custom_resoultion",
				_meta = "option"
			},
			{
				value = 2,
				ratio = 1/1,
				text_id = "1:1",
				localize = false,
				_meta = "option"
			},
			{
				value = 3,
				ratio = 4/3,
				text_id = "4:3",
				localize = false,
				_meta = "option"
			},
			{
				value = 4,
				ratio = 16/9,
				text_id = "16:9",
				localize = false,
				_meta = "option"
			},
			{
				value = 5,
				ratio = 16/10,
				text_id = "16:10",
				localize = false,
				_meta = "option"
			},
			{
				value = 6,
				ratio = 21/9,
				text_id = "21:9",
				localize = false,
				_meta = "option"
			},
			type = "MenuItemMultiChoice"
		}
		
		local new_item = new_node:create_item(data, params)
		new_node:add_item(new_item)
	end
	
	local function add_slider(name)
		if not new_node:item(name) then
			MenuCallbackHandler["res_"..name.."_callback"] = function(self, item)
				local aspect = Global.crimenet.res_aspect_ratio
				local ratio = aspect and new_node:item("res_aspect_ratio")._options[aspect]._parameters.ratio or false
				if name == "x_res_change" then
					if ratio then
						new_node:item("y_res_change"):set_value(math.round(item:value() / ratio))
					end
					new_node:item("x_res_change"):set_value(math.round(item:value()))
					new_node:item("change_res")._parameters.resolution = Vector3(new_node:item("x_res_change"):value(), new_node:item("y_res_change"):value(), nil)
				else
					if ratio then
						new_node:item("x_res_change"):set_value(math.round(item:value() * ratio))
					end
					new_node:item("y_res_change"):set_value(math.round(item:value()))
					new_node:item("change_res")._parameters.resolution = Vector3(new_node:item("x_res_change"):value(), new_node:item("y_res_change"):value(), nil)
				end
			end
			
			local params = {
				callback = "res_"..name.."_callback",
				name = name,
				text_id = name
			}
			
			local data = {
				type = "CoreMenuItemSlider.ItemSlider",
				show_value = true,
				max = name == "x_res_change" and max_res.x or max_res.y,
				min = 70,
				step = 1
			}
			
			local new_item = new_node:create_item(data, params)
			new_node:add_item(new_item)
			new_node:item(name):set_value(name == "x_res_change" and RenderSettings.resolution.x or RenderSettings.resolution.y)
		end
	end
	
	add_slider("x_res_change")
	add_slider("y_res_change")

	add_divider("res_change_divider", 10)
	
	MenuCallbackHandler.refresh_res_items = function(self, item)
		local resolution = item:parameters().resolution
		new_node:item("x_res_change"):set_value(resolution.x)
		new_node:item("y_res_change"):set_value(resolution.y)
	end
	
	if not new_node:item("change_res") then
		local params = {
			callback = "change_resolution_new",
			icon = "guis/textures/scrollarrow",
			icon_rotation = 90,
			icon_visible_callback = "is_current_custom_resolution",
			name = "change_res",
			text_id = "dialog_apply",
			resolution = Vector3(RenderSettings.resolution.x, RenderSettings.resolution.y, nil)
		}
	
		local new_item = new_node:create_item(nil, params)
		new_node:add_item(new_item)
	end
	
	add_divider("res_divider", 16)
	
	local modes = deep_clone(RenderSettings.modes)
	table.insert(modes, Vector3(854, 480, nil))
	table.insert(modes, Vector3(640, 480, nil))
	table.insert(modes, Vector3(640, 360, nil))
	table.insert(modes, Vector3(426, 240, nil))
	table.insert(modes, Vector3(320, 240, nil))
	table.insert(modes, Vector3(256, 144, nil))
	table.insert(modes, Vector3(128, 128, nil))

	table.sort(modes, function (a, b)
		return a.x > b.x
	end)

	table.sort(modes, function (a, b)
		return a.y > b.y
	end)
	
	for _, res in ipairs(modes) do
		local res_string = string.format("%d x %d", res.x, res.y)

		if not new_node:item(res_string) then
			local params = {
				localize = "false",
				icon = "guis/textures/scrollarrow",
				icon_rotation = 90,
				icon_visible_callback = "is_current_resolution",
				name = res_string,
				text_id = res_string,
				resolution = res
			}
			
			local data = {
				callback = "change_resolution_new refresh_res_items"
			}
			
			local new_item = new_node:create_item(data, params)
			new_node:add_item(new_item)
		end
	end
	
	new_node:item("res_aspect_ratio"):set_value(Global.crimenet.res_aspect_ratio or 1)
	
	managers.menu:add_back_button(new_node)
	
	return new_node
end

local data = MenuManager.show_accept_gfx_settings_dialog
function MenuManager:show_accept_res_dialog(func)
	data(self, func)
end

function MenuManager:show_accept_gfx_settings_dialog(func)
	local res = {
		RenderSettings.resolution.x,
		RenderSettings.resolution.y
	}

	if not res_allowed_fullscreen(res) then
		local full = (managers.viewport:is_fullscreen() or res[2] < 480) and 2
		set_windowed(true, full)
	end
end

MenuCallbackHandler.is_current_custom_resolution = function(self, item)
	local game_res = RenderSettings.resolution
	local item_res = item:parameters().resolution
	
	return (game_res.x == item_res.x and game_res.y == item_res.y) and not res_allowed_fullscreen(item_res)
end